# Code to accompany the paper "Evaluating measurement invariance 
#           in categorical data latent variable models with the EPC-interest"
# Take roll call data from the oc package and write it in format readable by LG
#
# Author: DL Oberski (daniel.oberski@gmail.com)
# Date: 2015-07-01


library(oc)
library(plyr)
library(reshape2)


data(sen90)

d <- sen90$votes
d <- cbind(d, party=NA)
d[rownames(sen90$legis.data), 'party'] <- as.character(sen90$legis.data$party)
d <- as.data.frame(d)
d$senator <- rownames(d)

d_long <- melt(d, id=c("senator","party"))
d_long <- arrange(d_long, senator)

codes <- unlist(sen90$codes)
names(codes) <- gsub("(.+?)\\d*", "\\1", names(codes))
codes_rev <- names(codes)
names(codes_rev) <- codes
codes_rev[codes_rev%in% c("notInLegis","missing")] <- NA
codes_rev[codes_rev=="yea"] <- 1
codes_rev[codes_rev=="nay"] <- 0

d_long$vote <- codes_rev[d_long$value]

d[,1:(ncol(d)-2)] <- as.numeric(codes_rev[unlist(d[,1:(ncol(d)-2)])])
names(d) <- gsub("Vote ", "B", names(d))

write.table(d, "~/Dropbox/Jeroen-Invariance/Data/roll_call_sen90.dat", na = ".", 
            col.names=TRUE, row.names=FALSE, quote=FALSE, sep="\t")



head(d_long)
names(d_long)[names(d_long)=="variable"] <- "bill"
write.table(d_long, "~/Dropbox/Jeroen-Invariance/Data/voting.dat", na = ".", 
            col.names=TRUE, row.names=FALSE, quote=FALSE, sep="\t")

